clear

# variables
VERSION=`cat version.txt`
ZIPVERSION="$VERSION"
SHOULDCLEAN="0"
SHOULDDISABLEUPDATE="0"
SHOULDSIGN="0"
SHOULDZIP="0"
SHOULDDMG="0"
SHOULDSOURCE="0"
SIGNCERT=""
ARCH=$(arch)

# detect the CLI flags
while [ "$1" != "" ]; do
  if [ "$1" == "-c" ]; then
    SHOULDCLEAN="1"
  elif [ "$1" == "-n" ]; then
    SHOULDDISABLEUPDATE="1"
  elif [ "$1" == "-u" ]; then
    ARCH="universal2"
  elif [ "$1" == "-cs" ]; then
    SHOULDSIGN="1"
    SIGNCERT="$2"
  elif [ "$1" == "-z" ]; then
    SHOULDZIP="1"
  elif [ "$1" == "-d" ]; then
    SHOULDDMG="1"
  elif [ "$1" == "-s" ]; then
    SHOULDSOURCE="1"
  fi

  shift
done

# cleanup
echo "Doing some cleanup...\n"
if [ "$SHOULDCLEAN" == "1" ]; then
  rm -rf build
fi
rm -rf dist

# build
echo "Executing the py2app build...\n"
python setup.py py2app --arch $ARCH --iconfile ./commit/assets/icons/appicon.icns

# should disable update?
if [ "$SHOULDDISABLEUPDATE" == "1" ]; then
  echo "Disable auto-update...\n"
  touch ./dist/Commit.app/Contents/Resources/noupdate
  ZIPVERSION="${VERSION}_noupdate"
fi

printf "Downloading Pandoc for inclusion in the app...\n"
python3 << EOF
import pypandoc
import os
target_path = os.path.abspath("build/")
pypandoc.pandoc_download.download_pandoc(
    download_folder=target_path,
    targetfolder=target_path
)
EOF
mkdir ./dist/Commit.app/Contents/Resources/bin
cp ./build/pandoc ./dist/Commit.app/Contents/Resources/bin

# sign the app
if [ "$SHOULDSIGN" == "1" ]; then
  echo "Signing the app for distribution...\n"
  mkdir -p ./dist/Commit.app/Contents/Helpers/dylibs
  mkdir -p ./dist/Commit.app/Contents/Helpers/dylibs2
  mkdir -p ./dist/Commit.app/Contents/Helpers/lib-dynload/AppKit
  mkdir -p ./dist/Commit.app/Contents/Helpers/lib-dynload/CoreFoundation
  mkdir -p ./dist/Commit.app/Contents/Helpers/lib-dynload/Foundation
  mkdir -p ./dist/Commit.app/Contents/Helpers/lib-dynload/objc
  mkdir -p ./dist/Commit.app/Contents/Helpers/lib-dynload/PIL/dylibs
  mkdir -p ./dist/Commit.app/Contents/Helpers/lib-dynload/yaml
  mkdir -p ./dist/Commit.app/Contents/Helpers/wx/svg
  mkdir -p ./dist/Commit.app/Contents/Helpers/numpy/core
  mkdir -p ./dist/Commit.app/Contents/Helpers/numpy/linalg
  mkdir -p ./dist/Commit.app/Contents/Helpers/numpy/dylibs
  mkdir -p ./dist/Commit.app/Contents/Helpers/numpy/fft
  mkdir -p ./dist/Commit.app/Contents/Helpers/numpy/random
  mv ./dist/Commit.app/Contents/Resources/zlib* ./dist/Commit.app/Contents/Helpers
  ditto -x -k ./dist/Commit.app/Contents/Resources/lib/python310.zip ./dist/Commit.app/Contents/Resources/lib/python310
  mv ./dist/Commit.app/Contents/Resources/lib/python310/PIL/.dylibs/*.dylib ./dist/Commit.app/Contents/Helpers/dylibs
  mv ./dist/Commit.app/Contents/Resources/lib/python3.10/lib-dynload/*.so ./dist/Commit.app/Contents/Helpers/lib-dynload
  mv ./dist/Commit.app/Contents/Resources/lib/python3.10/lib-dynload/AppKit/*.so ./dist/Commit.app/Contents/Helpers/lib-dynload/AppKit
  mv ./dist/Commit.app/Contents/Resources/lib/python3.10/lib-dynload/CoreFoundation/*.so ./dist/Commit.app/Contents/Helpers/lib-dynload/CoreFoundation
  mv ./dist/Commit.app/Contents/Resources/lib/python3.10/lib-dynload/Foundation/*.so ./dist/Commit.app/Contents/Helpers/lib-dynload/Foundation
  mv ./dist/Commit.app/Contents/Resources/lib/python3.10/lib-dynload/objc/*.so ./dist/Commit.app/Contents/Helpers/lib-dynload/objc
  mv ./dist/Commit.app/Contents/Resources/lib/python3.10/lib-dynload/PIL/*.so ./dist/Commit.app/Contents/Helpers/lib-dynload/PIL
  mv ./dist/Commit.app/Contents/Resources/lib/python3.10/lib-dynload/PIL/.dylibs/*.dylib ./dist/Commit.app/Contents/Helpers/lib-dynload/PIL/dylibs
  mv ./dist/Commit.app/Contents/Resources/lib/python3.10/lib-dynload/yaml/*.so ./dist/Commit.app/Contents/Helpers/lib-dynload/yaml
  mv ./dist/Commit.app/Contents/Resources/lib/python3.10/wx/*.dylib ./dist/Commit.app/Contents/Helpers/wx
  mv ./dist/Commit.app/Contents/Resources/lib/python3.10/wx/*.so ./dist/Commit.app/Contents/Helpers/wx
  mv ./dist/Commit.app/Contents/Resources/lib/python3.10/wx/svg/*.so ./dist/Commit.app/Contents/Helpers/wx/svg
  mv ./dist/Commit.app/Contents/Resources/lib/python3.10/numpy/core/*.so ./dist/Commit.app/Contents/Helpers/numpy/core
  mv ./dist/Commit.app/Contents/Resources/lib/python3.10/numpy/linalg/*.so ./dist/Commit.app/Contents/Helpers/numpy/linalg
  mv ./dist/Commit.app/Contents/Resources/lib/python3.10/numpy/.dylibs/*.dylib ./dist/Commit.app/Contents/Helpers/numpy/dylibs
  mv ./dist/Commit.app/Contents/Resources/lib/python3.10/numpy/fft/*.so ./dist/Commit.app/Contents/Helpers/numpy/fft
  mv ./dist/Commit.app/Contents/Resources/lib/python3.10/numpy/random/*.so ./dist/Commit.app/Contents/Helpers/numpy/random
  codesign -s "$SIGNCERT" -v --force --deep --timestamp --entitlements ./commit/assets/installer/mac/entitlements.plist -o runtime "./dist/Commit.app"
  mv ./dist/Commit.app/Contents/Helpers/dylibs/*.dylib ./dist/Commit.app/Contents/Resources/lib/python310/PIL/.dylibs/
  ditto -c -k ./dist/Commit.app/Contents/Resources/lib/python310 ./dist/Commit.app/Contents/Resources/lib/python310.zip
  mv ./dist/Commit.app/Contents/Helpers/zlib* ./dist/Commit.app/Contents/Resources/
  mv ./dist/Commit.app/Contents/Helpers/lib-dynload/*.so ./dist/Commit.app/Contents/Resources/lib/python3.10/lib-dynload
  mv ./dist/Commit.app/Contents/Helpers/lib-dynload/AppKit/*.so ./dist/Commit.app/Contents/Resources/lib/python3.10/lib-dynload/AppKit
  mv ./dist/Commit.app/Contents/Helpers/lib-dynload/CoreFoundation/*.so ./dist/Commit.app/Contents/Resources/lib/python3.10/lib-dynload/CoreFoundation
  mv ./dist/Commit.app/Contents/Helpers/lib-dynload/Foundation/*.so ./dist/Commit.app/Contents/Resources/lib/python3.10/lib-dynload/Foundation
  mv ./dist/Commit.app/Contents/Helpers/lib-dynload/objc/*.so ./dist/Commit.app/Contents/Resources/lib/python3.10/lib-dynload/objc
  mv ./dist/Commit.app/Contents/Helpers/lib-dynload/PIL/*.so ./dist/Commit.app/Contents/Resources/lib/python3.10/lib-dynload/PIL
  mv ./dist/Commit.app/Contents/Helpers/lib-dynload/PIL/dylibs/*.dylib ./dist/Commit.app/Contents/Resources/lib/python3.10/lib-dynload/PIL/.dylibs
  mv ./dist/Commit.app/Contents/Helpers/lib-dynload/yaml/*.so ./dist/Commit.app/Contents/Resources/lib/python3.10/lib-dynload/yaml
  mv ./dist/Commit.app/Contents/Helpers/wx/*.dylib ./dist/Commit.app/Contents/Resources/lib/python3.10/wx
  mv ./dist/Commit.app/Contents/Helpers/wx/*.so ./dist/Commit.app/Contents/Resources/lib/python3.10/wx
  mv ./dist/Commit.app/Contents/Helpers/wx/svg/*.so ./dist/Commit.app/Contents/Resources/lib/python3.10/wx/svg
  mv ./dist/Commit.app/Contents/Helpers/numpy/core/*.so ./dist/Commit.app/Contents/Resources/lib/python3.10/numpy/core
  mv ./dist/Commit.app/Contents/Helpers/numpy/linalg/*.so ./dist/Commit.app/Contents/Resources/lib/python3.10/numpy/linalg
  mv ./dist/Commit.app/Contents/Helpers/numpy/dylibs/*.dylib ./dist/Commit.app/Contents/Resources/lib/python3.10/numpy/.dylibs
  mv ./dist/Commit.app/Contents/Helpers/numpy/fft/*.so ./dist/Commit.app/Contents/Resources/lib/python3.10/numpy/fft
  mv ./dist/Commit.app/Contents/Helpers/numpy/random/*.so ./dist/Commit.app/Contents/Resources/lib/python3.10/numpy/random
  rm -r ./dist/Commit.app/Contents/Helpers
  rm -r ./dist/Commit.app/Contents/Resources/lib/python310
  codesign -s "$SIGNCERT" -v --force --deep --timestamp --entitlements ./commit/assets/installer/mac/entitlements.plist -o runtime "./dist/Commit.app"
fi

# create DMG file
if [ "$SHOULDDMG" == "1" ]; then
  echo "Creating the DMG file for distribution...\n"

  # generate the license RTF file for the DMG
  echo Creating the license RTF file...
  cd build
  ./pandoc -f commonmark -t rtf -s -o LICENSE.rtf ../LICENSE.md
  cd ..

  # copy the webloc file (want different names in source vs. package)
  cp ./commit/assets/installer/mac/dmg-manual.webloc ./build/Manual.webloc

  # build the "delete settings" script
  osacompile -o "./dist/Reset Settings.app" ./commit/assets/installer/mac/dmg-delete-settings.applescript

  # prepare the signing string for the JSON if -CS flag is present
  # also sign the Reset Settings app/script
  DMGSIGNSTRING=""
  if [ "$SHOULDSIGN" == "1" ]; then
    DMGSIGNSTRING=",\"code-sign\": {\"signing-identity\":\"$SIGNCERT\"}"
    codesign -s "$SIGNCERT" -v --force --deep --timestamp -o runtime "./dist/Reset Settings.app"
  fi

  # build the appdmg json definition
  cat << EOF > build/commit-dmg.json
{
  "title": "Commit",
  "icon": "../commit/assets/installer/mac/dmg-icon.icns",
  "background": "../commit/assets/installer/mac/dmg-background.png",
  "window": {
    "size": {
      "width": 620,
      "height": 650
    }
  },
  "contents": [
    { "x": 165, "y": 381, "type": "file", "path": "../dist/Commit.app" },
    { "x": 455, "y": 381, "type": "link", "path": "/Applications" },
    { "x": 195, "y": 525, "type": "file", "path": "../build/LICENSE.rtf" },
    { "x": 310, "y": 525, "type": "file", "path": "../build/Manual.webloc" },
    { "x": 425, "y": 525, "type": "file", "path": "../dist/Reset Settings.app" }
  ]$DMGSIGNSTRING
}
EOF

  # run appdmg builder
  appdmg ./build/commit-dmg.json ./dist/commit_$ZIPVERSION.dmg
fi

# create ZIP file
if [ "$SHOULDZIP" == "1" ]; then
  echo "Creating the ZIP file(s) for distribution...\n"
  ditto -c -V -k --keepParent ./dist/Commit.app ./dist/commit_${ZIPVERSION}_mac.zip

  # if we're code signing, go ahead and sign the ZIP. Why not?
  if [ "$SHOULDSIGN" == "1" ]; then
    codesign -s "$SIGNCERT" "./dist/commit_${ZIPVERSION}_mac.zip"
  fi

  # if we had made a DMG, we have a reset app we need to ZIP for notarization too
  if [ "$SHOULDDMG" == "1" ]; then
      ditto -c -V -k --keepParent "./dist/Reset Settings.app" ./dist/commit_reset_${ZIPVERSION}_mac.zip
      if [ "$SHOULDSIGN" == "1" ]; then
        codesign -s "$SIGNCERT" "./dist/commit_reset_{$ZIPVERSION}_mac.zip"
      fi
  fi
fi

# create a source tarball
if [ "$SHOULDSOURCE" == "1" ]; then
  echo "Creating the source tarball...\n"
  tar --exclude='.git' --exclude='.idea' --exclude='__pycache__' --exclude='./build' --exclude='./dist' --exclude='./venv' -czvf ./dist/commit_${VERSION}_source.tar.gz ./
fi

# done
echo "Our work here is done.\n"
