# This is a configuration file for the Nullsoft Scriptable Install System
# https://nsis.sourceforge.io/

# Use the Modern NSIS UI and nsDialogs
!include "MUI2.nsh"
!include "nsDialogs.nsh"

# Installer name
Name "Commit"

# Installer file name (%commitver% set in build-windows.cmd from version.txt)
Outfile "dist\commit_$%commitbuildver%.exe"

# Enable Unicode and DPI Awareness
Unicode True
ManifestDPIAware True

# Default install directory
InstallDir "$PROGRAMFILES64\Commit\"

# Installer text
DirText "This will install Commit on your computer. Choose the installation directory."

# Variables
!define REGNAME "IntersanityCommit"
!define EXENAME "commit.exe"
!define REGFILEEXT ".ctxt"
!define REGPATH_UNINSTSUBKEY "Software\Microsoft\Windows\CurrentVersion\Uninstall\${REGNAME}"
!define REGPATH_FILETYPEKEY "Software\Classes\${REGNAME}"
!define REGPATH_ASSOCEXE "Software\Classes\Applications\${EXENAME}"
!define LOCKSCAN_TITLE "Scanning for Open and Locked Files"
!define LOCKSCAN_SUBTITLE "This will take a few moments. Thank you for your patience!"

# Override InstallDir if we have an existing install location (i.e., reinstall/upgrade)
InstallDirRegKey HKLM "${REGPATH_UNINSTSUBKEY}" "InstDir"

# EXE metadata
VIProductVersion "$%commitwinver%"
VIAddVersionKey "FileVersion" "$%commitwinver%"
VIAddVersionKey "ProductName" "Commit"
VIAddVersionKey "FileDescription" "Commit application installer"
VIAddVersionKey "CompanyName" "Intersanity Enterprises"
VIAddVersionKey "LegalCopyright" "(c) 2022-2023 Intersanity Enterprises and contributors"

# MUI installer settings flags
!define MUI_ABORTWARNING
!define MUI_ICON "commit\assets\installer\win\nsis-icon.ico"
!define MUI_WELCOMEFINISHPAGE_BITMAP "commit\assets\installer\win\nsis-sidebar.bmp"
!define MUI_FINISHPAGE_RUN "commit.exe"
!define MUI_FINISHPAGE_RUN_TEXT "Launch Commit now"
!define MUI_FINISHPAGE_RUN_NOTCHECKED
!define MUI_FINISHPAGE_LINK "Online User Manual"
!define MUI_FINISHPAGE_LINK_LOCATION "https://www.intersanity.com/software/commit/manual/"

# MUI uninstaller settings flags
!define MUI_UNABORTWARNING
!define MUI_UNICON "commit\assets\installer\win\nsis-unicon.ico"
!define MUI_UNWELCOMEFINISHPAGE_BITMAP "commit\assets\installer\win\nsis-sidebar.bmp"

# Installer Pages
!insertmacro MUI_PAGE_WELCOME
!insertmacro MUI_PAGE_LICENSE "build\LICENSE.rtf"
!insertmacro MUI_PAGE_DIRECTORY
Page Custom LockedList
!insertmacro MUI_PAGE_INSTFILES
!insertmacro MUI_PAGE_FINISH

# Uninstaller Pages
!define MUI_PAGE_CUSTOMFUNCTION_SHOW un.AddSettingClearCheckbox
!define MUI_PAGE_CUSTOMFUNCTION_LEAVE un.ReadSettingClearCheckbox
!insertmacro MUI_UNPAGE_WELCOME
UninstPage Custom un.LockedList
!insertmacro MUI_UNPAGE_INSTFILES
!insertmacro MUI_UNPAGE_FINISH

# Language settings
!insertmacro MUI_LANGUAGE "English"

# put a "clear settings" option on installer
Var SettingClearCheckbox
Var SettingClearCheckboxValue
Function un.AddSettingClearCheckbox
    # build the checkbox and pop it into the page
    ${NSD_CreateCheckbox} 36% -18u 50% 12u "Delete my Commit settings"
    Pop $SettingClearCheckbox
    SetCtlColors $SettingClearCheckbox "" "${MUI_BGCOLOR}"
FunctionEnd
Function un.ReadSettingClearCheckbox
    # read the checkbox value and save it to a variable
    ${NSD_GetState} $SettingClearCheckbox $SettingClearCheckboxValue
FunctionEnd

# Check if the files are locked (installer version)
Function LockedList
    !insertmacro MUI_HEADER_TEXT "${LOCKSCAN_TITLE}" "${LOCKSCAN_SUBTITLE}"

    # see if we have an old installation; if so, add it to the lock checker (i.e., clear the way for uninstall)
    ReadRegStr $R0 HKLM "${REGPATH_UNINSTSUBKEY}" "InstDir"
    ${If} $R0 != ""
        LockedList::AddModule '$R0\commit.exe'
    ${EndIf}

    # check the selected directory for a running copy of Commit
    LockedList::AddModule "$INSTDIR\commit.exe"

    # pop the dialog; will continue automatically if no locks or when locks closed
    LockedList::Dialog /autonext
    Pop $R1
FunctionEnd

# Check if the files are locked (uninstaller version)
Function un.LockedList
    !insertmacro MUI_HEADER_TEXT "${LOCKSCAN_TITLE}" "${LOCKSCAN_SUBTITLE}"

    # all we're doing is checking if the app is running from the uninstall dir
    LockedList::AddModule "$INSTDIR\commit.exe"

    # pop the dialog; tries to close silently and continue automatically
    LockedList::Dialog /autonext /autoclose "" "" "" ""
    Pop $R0
FunctionEnd

# Install section
Section "Install"
    SetOutPath $INSTDIR

    # detect if we have an old installation
    ReadRegStr $R0 HKLM "${REGPATH_UNINSTSUBKEY}" "QuietUninstallString"
    ReadRegStr $R1 HKLM "${REGPATH_UNINSTSUBKEY}" "InstDir"

    # if we have an old installation, we want to run the uninstaller to remove it
    # this cleans things up, allows the user to change their install dir, etc.
    ${If} $R0 != ""
        # by default, NSIS uninstallers copy to temp dir and run from there (so it can delete itself)
        # that causes problems in this scenario, since it results in a grandchild process that ExecWait isn't aware of
        # when that happens, reinstall in same dir happens async with uninstall and things get dumb and messed up
        ExecWait '$R0 _?=$R1'

        # because of the above, it can't delete itself or the directory...so do that now that the run is complete
        Delete '$R1\Uninstall.exe'

        # now delete the dir (if empty) since the uninstaller couldn't do it because of the in-place run described above
        RMDir '$R1'
    ${EndIf}

    # get all files in dist
    File /r "dist\commit\*.*"

    # make an uninstaller
    WriteUninstaller "$INSTDIR\Uninstall.exe"

    # register uninstall info in the registry
    WriteRegStr HKLM "${REGPATH_UNINSTSUBKEY}" "DisplayName" "Commit: the Advanced Modern Typewriter"
    WriteRegStr HKLM "${REGPATH_UNINSTSUBKEY}" "Publisher" "Intersanity Enterprises"
    WriteRegStr HKLM "${REGPATH_UNINSTSUBKEY}" "InstDir" "$INSTDIR"
    WriteRegStr HKLM "${REGPATH_UNINSTSUBKEY}" "UninstallString" "$\"$INSTDIR\Uninstall.exe$\""
    WriteRegStr HKLM "${REGPATH_UNINSTSUBKEY}" "QuietUninstallString" "$\"$INSTDIR\Uninstall.exe$\" /S"
    WriteRegStr HKLM "${REGPATH_UNINSTSUBKEY}" "DisplayIcon" "$\"$INSTDIR\${EXENAME}$\",0"
    WriteRegStr HKLM "${REGPATH_UNINSTSUBKEY}" "DisplayVersion" "$%commitver%"
    WriteRegDWORD HKLM "${REGPATH_UNINSTSUBKEY}" "NoModify" "1"
    WriteRegDWORD HKLM "${REGPATH_UNINSTSUBKEY}" "NoRepair" "1"

    # start menu shortcut
    CreateShortCut "$SMPROGRAMS\Commit.lnk" "$INSTDIR\commit.exe"

    # file type associations
    WriteRegStr HKLM "${REGPATH_FILETYPEKEY}\DefaultIcon" "" "$\"$INSTDIR\commit.exe$\",1"
    WriteRegStr HKLM "${REGPATH_FILETYPEKEY}\shell\open\command" "" "$\"$INSTDIR\commit.exe$\" $\"%1$\""
    WriteRegStr HKLM "Software\Classes\${REGFILEEXT}" "" "${REGNAME}"
    WriteRegStr HKLM "Software\Classes\${REGFILEEXT}" "Content Type" "text/com.intersanity.commit"
    WriteRegStr HKLM "Software\Classes\${REGFILEEXT}" "PerceivedType" "text"
    WriteRegStr HKLM "${REGPATH_ASSOCEXE}\shell\open\command" "" "$\"$INSTDIR\commit.exe$\" $\"%1$\""
    WriteRegStr HKLM "${REGPATH_ASSOCEXE}" "FriendlyAppName" "Commit"
    WriteRegStr HKLM "${REGPATH_ASSOCEXE}" "ApplicationCompany" "Intersanity Enterprises"
SectionEnd

# Uninstall section
Section "Uninstall"
    # remove self (note won't work when running uninstall from install)
    Delete "$INSTDIR\Uninstall.exe"

    # remove subdirectories
    RMDir /r "$INSTDIR\commit"
    RMDir /r "$INSTDIR\lib"

    # open the uninstlog file - list of installed files in root dir
    # we delete root files BY NAME to avoid 'RMDir /r "$INSTDIR"' which is BAD (see docs)
    FileOpen $0 "$INSTDIR\uninstlog.txt" r

    LOOP:
        # reset any old errors; if new one jump to end
        ClearErrors
        IfErrors ENDLOOP

        # read a line from the file list
        FileRead $0 $1

        # remove /r/n from end of line (eyeroll)
        StrCpy $1 $1 -2

        # if we got a blank line, jump to the end
        ${If} $1 == ""
            Goto ENDLOOP
        ${EndIf}

        # delete this particular file
        Delete "$INSTDIR\$1"

        # back to start of loop
        Goto LOOP
    ENDLOOP:

    # close and remove the uninstlog file (we're done with it)
    FileClose $0
    Delete "$INSTDIR\uninstlog.txt"

    # try to remove the directory (will silently fail if not empty)
    RMDir "$INSTDIR"

    # remove uninstaller reference
    DeleteRegKey HKLM "${REGPATH_UNINSTSUBKEY}"

    # remove file associations
    DeleteRegKey HKLM "${REGPATH_FILETYPEKEY}"
    DeleteRegKey HKLM "Software\Classes\${REGFILEEXT}"
    DeleteRegKey HKLM "${REGPATH_ASSOCEXE}"

    # remove start menu shortcut
    Delete "$SMPROGRAMS\Commit.lnk"

    # and finally, reset settings if option was selected
    ${If} $SettingClearCheckboxValue == 1
        RMDir /r "$LOCALAPPDATA\intersanity\commit\"
        RMDir "$LOCALAPPDATA\intersanity\"
    ${EndIf}

SectionEnd
